/************************************************************************
*																		*
*	TextImport.c														*
*																		*
*	Translator for reading a text file with XTND 1.3.  This translator	*
*	does not support headers, footers, pictures, or footnotes, since	*
*	it reads only simple, unstructured ASCII files.						*
*																		*
*	Copyright  1989,90 Claris Corporation								*
*	All Rights Reserved													*
*																		*
*	Author: Lyndley Crumly												*
*	Date:   12 February, 1990											*
*																		*
************************************************************************/


/**** Note: TextExport. MUST be compiled and merged after this		****/
/**** project is compiled, since this project replaces the 			****/
/**** resources in the Text translator when it is built, including	****/
/**** the resource created by TextExport..							****/

#include <StandardFile.h>
#include <Fonts.h>
#include <Memory.h>
#include <Printing.h>
enum { FALSE, TRUE };

#include ":::XTND Headers:XTNDCIncludes:XTNDTextTranslator.h"


/*------------------------- Useful Constants ---------------------------*/

#define kTempBufferSize	512			/* Text buffer size					*/
#define kFixedInch 		0x00480000	/* 72 pixels in fixed format		*/
#define kSpaceChar 		0x20		/* Space character in hex			*/


/*-------------------------Global Variables-----------------------------*/

/* The translator globals structure is used to store our import 		*/
/* translators translators private global variables. A relocatable 	*/
/* block is created in the application heap when the translator 		*/
/* receives the IMPORT_INIT_ALL directive and a handle to the globals 	*/
/* are stored in the GlobalHandle field of the Import Parameter Block 	*/
/* passed into the translator from the application. The globals block 	*/
/* is freed when the translator receives the IMPORT_CLOSE_ALL directive.*/

/* We are going to lock down the global handle when it is created and	*/
/* leave it that way so as to avoid any problems with dereferenced		*/
/* pointers.  You may choose to unlock the global handle when feasible.	*/

typedef struct TransGlobals {
	unsigned char 	tempBuffer[kTempBufferSize];	/* Memory buffer where data from the disk file is stored */
	Ptr				outPtr;						/* Pointer into the text buffer which is passed to the application */
	long			EOFPosition;				/* This is used to tell when we have reached the end of the data file */
	long			bufferStart;				/* This stores the file position at which we started loading data */
	long			charPos;					/* This variable always indicates the next character to be read from the input data file */
	Boolean 		firstTime;					/* Flag used to indicate whether this the first GetText call.  If it is the first GetText, then some initialization is done to set up the parameter block for subsequent GetText calls.					*/
} TMyGlobals, *TMyGlobalPtr, **TMyGlobalHandle;


/*-------------------------Function prototypes--------------------------*/

		void main(ImportParmBlkPtr importPBPtr);
static	void InitAll(ImportParmBlkPtr importPBPtr, TMyGlobalHandle *myGlobals);
static	void InitMain(ImportParmBlkPtr importPBPtr, TMyGlobalPtr myGlobalPtr);
static	void GetText(ImportParmBlkPtr importPBPtr, TMyGlobalPtr myGlobalPtr);
static	unsigned char GetNextChar(ImportParmBlkPtr importPBPtr, TMyGlobalPtr myGlobalPtr);
static	void FillTempBuffer(long filepos, ImportParmBlkPtr importPBPtr, TMyGlobalPtr myGlobalPtr);


/*----------------------------------------------------------------------*/
/*	main	This is the main routine and the only entry point for the	*/
/*			the translator.  This routine actually performs all of the	*/
/*			work of the translator.  It dispatches the call to the 		*/
/*			appropriate routine depending on the directive and 			*/
/*			translator state.											*/
/*----------------------------------------------------------------------*/
void main(register ImportParmBlkPtr importPBPtr)
{
	switch(importPBPtr->directive)
	{
		case importInitAll:		/* Initialize the translator.  This		*/
									/* must be the first call to the 		*/
									/* must be the first call to the 		*/
									/* translator.							*/
			InitAll(importPBPtr, (TMyGlobalHandle *)&importPBPtr->globalHandle);
			break;

		case importInitMain:		/* Initialize the main story.			*/
									/* Nothing needs to be done here.		*/
			importPBPtr->directive = importAcknowledge;
			break;

		case importGetText:		/* Get the next text run from the main	*/
									/* body, since that is all we import.	*/
			GetText(importPBPtr, (TMyGlobalPtr)*importPBPtr->globalHandle);
			break;

		case importCloseAll:		/* Shutdown the translator.  This must	*/
									/* be the last call to the translator.	*/
			if (importPBPtr->globalHandle) {
				DisposHandle(importPBPtr->globalHandle);
				(importPBPtr->globalHandle) = 0L;		/* nil */
			}
			break;

		default:					/* We don't show the directives that we */
			break;					/* don't use. They are all taken care   */
									/* of by default, which does nothing    */
									/* at all.								*/
	}
}


/*----------------------------------------------------------------------*/
/*	InitAll	This routine performs all initialization for the			*/
/*			translator.  This must be the first call to the translator,	*/
/*			although this particular translator does not check to make	*/
/*			sure that this call was the first one made.					*/
/*----------------------------------------------------------------------*/
static void InitAll(register ImportParmBlkPtr importPBPtr, 
					register TMyGlobalHandle *myGlobals)
{
	/* First try to create a global handle and quit if we can't */

	if (!(*myGlobals = (TMyGlobalHandle)NewHandle(sizeof(TMyGlobals)))) {
		importPBPtr->result = MemError();
		return;
	}

	/* Save the global handle in the field provided in the parameter block */
	MoveHHi((Handle)*myGlobals);
	HLock((Handle)*myGlobals);
	importPBPtr->globalHandle = (Handle)*myGlobals;

	/* find the EOF position of the input file */
	if (importPBPtr->result = GetEOF(importPBPtr->refNum, &(***myGlobals).EOFPosition))
		return;													/* can't do anything about this	*/
	(***myGlobals).bufferStart = (***myGlobals).EOFPosition;	/* So we know the buffer is empty	*/
	(***myGlobals).charPos = 0L;								/* Start reading at the beginning of the file	*/
	(***myGlobals).firstTime = TRUE;

	/* Set initial values for fields in the import parameter block.		*/
	/* Some of these are purely arbitrary, and may be changed at your	*/
	/* discretion.														*/

	importPBPtr->numCols = 1;
	importPBPtr->autoHyphenate = FALSE;
	importPBPtr->topMargin = kFixedInch;
	importPBPtr->bottomMargin = kFixedInch;
	importPBPtr->leftMargin = kFixedInch;
	importPBPtr->rightMargin = kFixedInch;
	importPBPtr->gutter = 0x000C0000;
	importPBPtr->startPageNum= 1;
	importPBPtr->startFootnoteNum= 1;

	importPBPtr->rulerShowing= TRUE;
	importPBPtr->doubleSided = FALSE;
	importPBPtr->titlePage = FALSE;
	importPBPtr->endnotes = FALSE;
	importPBPtr->showInvisibles = FALSE;
	importPBPtr->showPageGuides = FALSE;
	importPBPtr->showPictures = TRUE;
	importPBPtr->autoFootnotes = TRUE;
	importPBPtr->smartQuotes = FALSE;
	importPBPtr->fractCharWidths = FALSE;
}


/*----------------------------------------------------------------------*/
/*	InitMain This routine performs some initialization when the first	*/
/* 			GetText call is made, and not when the InitMain call is 	*/
/*			made, as you might assume from the name.  The reason for	*/
/* 			this is that these fields cannot be set before the first	*/
/*			GetText call, as they could be changed by the application.	*/
/*----------------------------------------------------------------------*/
static void InitMain(register ImportParmBlkPtr importPBPtr, 
					 register TMyGlobalPtr myGlobalPtr)
{
	/* Set initial values for fields in the import parameter block.		*/
	/* Some of these are purely arbitrary, and may be changed at your	*/
	/* discretion.														*/

	importPBPtr->txtFace = textPlain;
	importPBPtr->txtSize = 12;
	importPBPtr->txtFont = applFont;
	importPBPtr->txtColor = textBlack;
	importPBPtr->txtJust = textLeft;

	importPBPtr->paraFmts[0] = 0;				/* Left Paragraph Margin	*/
	importPBPtr->paraFmts[1] = 0;				/* Left Paragraph Indent	*/
	importPBPtr->paraFmts[2] = 0;				/* Right Paragraph Margin	*/
	importPBPtr->paraFmts[3] = 0;				/* Single Line Spacing		*/
	importPBPtr->paraFmts[4] = 0;				/* No Space Before			*/
	importPBPtr->paraFmts[5] = 0;				/* No Space After			*/
	importPBPtr->paraFmts[6] = -1;				/* Fixed Line Spacing		*/
	importPBPtr->paraFmts[7] = 0;				/* Fixed Point Spacing		*/
	importPBPtr->paraFmts[8] = 0;				/* Fixed Point Spacing		*/

	importPBPtr->tabs[0].tabIndent = -1;		/* First tab setting, which	*/
												/* means there are no tabs.	*/

	myGlobalPtr->firstTime = FALSE;			/* Don't come back here again	*/
}


/*----------------------------------------------------------------------*/
/*	GetText	This routine reads the next text run and returns it to the	*/
/*			application.
/*----------------------------------------------------------------------*/
static void GetText(register ImportParmBlkPtr importPBPtr, 
					register TMyGlobalPtr myGlobalPtr)
{
	unsigned char ch;			/* The next character from the text buffer	*/
	short count = 0;			/* The number of characters we will return	*/
	Boolean	endOfRun = FALSE;	/* Indicates the end of the text run	*/

	if (myGlobalPtr->firstTime)	/* Initialize text fields on the first GetText call	*/
			InitMain(importPBPtr, myGlobalPtr);

	/* Set up a pointer into our output buffer.  This pointer is 		*/
	/* incremented as each character is added to the buffer.			*/

	myGlobalPtr->outPtr = importPBPtr->textBuffer;

	/* Read characters until we reach the end of the run, which has a	*/
	/* maximum length of 255 characters and must end at the end of the	*/
	/* file (just to make things simpler).								*/

	while (myGlobalPtr->charPos < myGlobalPtr->EOFPosition && count < 255 && !endOfRun) {
		if (importPBPtr->result) {				/* If we got a read error,	*/
			count = -1;							/* Terminate the import		*/
			break;
		}

		ch = GetNextChar(importPBPtr, myGlobalPtr);	/* Retrieve the next character	*/

		/* Characters below the space character are not returned, with	*/
		/* the exception of the space, new page and return characters.	*/
		
		if (ch < kSpaceChar) {
			if (ch == tabChar || ch == newPage || ch == returnChar) {
				*(myGlobalPtr->outPtr) = ch;		/* Return these characters unchanged	*/
				(myGlobalPtr->outPtr)++;
				count++;
			}
			if (ch == returnChar || ch == newPage)
				endOfRun = TRUE;	/* These characters end a text run	*/

		/* All other characters are returned unchanged, except for 		*/
		/* 0x7F, which is not a valid character in the Macintosh 		*/
		/* character set. (It's a delete character or something.)		*/

		} else {
			if (ch != 0x7F) {
				*(myGlobalPtr->outPtr) = ch;
				(myGlobalPtr->outPtr)++;
				count++;
			}
		}
	}

	if (count <= 0)					/* If no data was found to be returned	*/
		 importPBPtr->directive = importAcknowledge;		/* End of data	*/

	importPBPtr->textLength = count;
}


/*----------------------------------------------------------------------*/
/*	GetNextChar	This routine retrieves the next character from the text	*/
/*			buffer and returns it, automatically incrementing charPos.	*/
/*----------------------------------------------------------------------*/
static unsigned char GetNextChar(register ImportParmBlkPtr importPBPtr, 
								 register TMyGlobalPtr myGlobalPtr)
{
	short bufferpos;

	if (myGlobalPtr->charPos >= myGlobalPtr->EOFPosition)			/* If we are reading past end of file	*/
		return(-1);													/* Return some arbitrary invalid character	*/

	bufferpos = myGlobalPtr->charPos - myGlobalPtr->bufferStart;	/* Find our position in buffer	*/

	/* If bufferpos is outside the buffer, then the buffer does not		*/
	/* contain the requested character.  The buffer will be refilled	*/
	/* from the file starting at the requested character.				*/
	
	if ((bufferpos < 0) || (bufferpos >= kTempBufferSize)) {
		FillTempBuffer(myGlobalPtr->charPos, importPBPtr, myGlobalPtr);	/* Fill the buffer from the file	*/
		myGlobalPtr->bufferStart = myGlobalPtr->charPos;
		bufferpos = 0;
	}
	myGlobalPtr->charPos++;							/* Increment charPos for next call	*/

	return((myGlobalPtr->tempBuffer)[bufferpos]);	/* Return the requested character	*/
}


/*----------------------------------------------------------------------*/
/*	FillTempBuffer	This routine fills the temporary buffer starting	*/
/* 				at the given position in the data file.					*/
/*----------------------------------------------------------------------*/
static void FillTempBuffer(long filepos, 
						   register ImportParmBlkPtr importPBPtr, 
						   register TMyGlobalPtr myGlobalPtr)
{
	long bytes;
	short readcount;

	if (importPBPtr->result) return;				/* Cancel if a previous error exists	*/
	readcount = kTempBufferSize ;					/* Use constant to set buffer size	*/

	if (myGlobalPtr->EOFPosition < filepos + readcount)		/* Make sure we don't read	*/
		readcount = myGlobalPtr->EOFPosition - filepos;		/* past the end of file		*/

	bytes = readcount;

	/* Set the file position at which to start reading */
	if (importPBPtr->result = SetFPos(importPBPtr->refNum, fsFromStart, filepos)) return;

	/* Read the data into the temporary buffer */
	importPBPtr->result = FSRead(importPBPtr->refNum, &bytes, myGlobalPtr->tempBuffer);
}
