/************************************************************************
*																		*
*	XTNDInterface.h														*
*																		*
*	Claris XTND Architecture application header file					*
*																		*
*	Copyright  1989-91 Claris Corporation								*
*	All Rights Reserved													*
*																		*
************************************************************************/

#ifndef __XTNDINTERFACE__
#define	__XTNDINTERFACE__

#ifdef applec
#	include <Types.h>
#	include <StandardFile.h>
#endif

/*  Prototypes for XTND system calls are at the end of this file (for THINK C)  */


/*  Defines for XTND Version  */

#define	xtndVersion		2


/*  Defines for file types  */

#define	xtndTranslatorFileType	'Fltr'


/*  Defines for resource types  */

#define	translatorDescription		'FTYP'
#define	textImportType				'FLTI'
#define	textExportType				'FLTE'
#define	pictImportType				'PFLT'
#define ClarisWorksDBImportType		'CWDI'
#define ClarisWorksDBExportType		'CWDE'
#define ClarisWorksSSImportType		'CWSI'
#define ClarisWorksSSExportType		'CWSE'
#define ClarisWorksBothImportType	'CWBI'
#define ClarisWorksBothExportType	'CWBE'


/*  Characteristics values for Flags field in TransDescribe and FTYP  */

#define	ftypIsSpecial			0x0001
#define	ftypHasPreferences		0x0002	/* Reserved for future use */
#define	ftypNeedsResources		0x0004
#define	ftypWritesResources		0x0008	/* Reserved for future use */
#define	ftypOnlyPreferences		0x0010	/* Reserved for future use */
#define	ftypIsLocalized			0x0020


/*  Type of translators to be included in list  */

#define	allowTypeMask			0x00FF
#define	allowText				0x0001
#define	allowGraphics			0x0002	/* Reserved for future use */
#define	allowDataBase			0x0004	/* Reserved for future use */
#define	allowPict				0x0008
#define	allowClarisWorks		0x0010	/* Reserved for ClarisWorks */
#define	allowOtherTypes			0x0080	/* Reserved for future use */
#define	allowAllTypes			0x00FF


/*  Use of translators to be included in list  */

#define	allowUseMask			0xFF00
#define	allowImport				0x0100
#define	allowExport				0x0200
#define	allowOtherUses			0x8000	/* Reserved for future use */
#define	allowAllUses			0xFF00


/*  Error codes which the XTND System can return  */

#define	badXTNDVersionErr		-9991
#define	noTransMatchErr			-9999


/*  Error codes which a translator can return  */

#define	badImportFileErr		12
#define	translationCanceledErr	32767


/*   T Y P E   D E F I N I T I O N S   */

#ifndef __XTND__

#ifdef THINK_C
	typedef short (* XTNDDlgHookProcPtr)();		/* Have to call this with CallPascalW() in Think C */

	#ifndef ModalFilterProcPtr	/* Define this for Think C users who don't already have it defined */
	#define	ModalFilterProcPtr	ProcPtr
	#endif
#else
	typedef pascal short (*XTNDDlgHookProcPtr)(short theItem, DialogPtr theDialog, Ptr theSFParams, Boolean *changedFlag, long unused);
#endif


/*  Translator procedure pointer.  Used for calling external code resources  */	

typedef	void (*TransProcPtr) (void *paramBlk);


/*  TransDescribe and Related Structures  */

/* The MatchInfo data stucture defines a single file type (and possibly creator type)	*/
/* that a translator can recognize.														*/
struct MatchInfo
{
	ResType			docCreator;			/* Document creator, eg MACA - MacWrite	*/
	ResType			docType;			/* Document type, eg WORD - MacWrite	*/
	Boolean			exactMatch;			/* If TRUE, Creator MUST match			*/
	Byte			creatorAndTypeMask;	/* Flags telling which bytes of creator & file type matter; creator is bits 0 - 3 and file type is bits 4 - 7	*/
};
typedef struct MatchInfo MatchInfo;


/* The TransDescribe data stucture defines an import/export translator.  There is one	*/
/* of these (corresponds to the 'FTYP' resource) for every import/export translator.	*/
struct TransDescribe
{
	short			version;		/* Version of this FTYP (currently 2) */
	ResType			translatorType;	/* Type of translator (ie, 'PFLT' for pictures) */
	short			codeResID;		/* Resource number of translator code resource */
	short			FDIFResID;		/* Resource ID of 'FDIF' (or -1 if none) */
	short			numVersBytes;	/* Number of bytes for version check (zero if no check) */
	long			versBytesOffSet;/* Offset into file for version check */
	unsigned char	versBytes[16];	/* Byte values we are looking for (max 16) */
	short			appWDRefNum;	/* Internal XTND use only - WDRefNum of the directory containing the application */
	short			unused1;		/* Reserved for future use - must be zero for this version */
	short			pathLength;		/* Internal XTND use only - Number of times this file has been translated */
	short			flags;			/* Flags for filter characteristics */
									/*	bit 0 - set if this filter won't work with All Available */
									/*	bit 1 - reserved for future use - must be zero for this version */
									/*	bit 2 - set if this filter must be able to read its resources */
									/*	bit 3 - 4 reserved for future use - must be zero for this version */
									/*	bit 5 - set if this filter is localized to a particular language */
									/* 	bit 6 - 13 reserved for future use - must be zero for this version */
									/*  bit 14 - 15 Used internally - must be zero for this version */
	short			transIndex;		/* Internal XTND use only - Index of this translator in the list stored by the FINI */
	short			resRefNum;		/* RefNum of resource fork of the translator file when open */
	long			directoryID;	/* Directory ID of the folder containing the filter */
	short			vRefNum;		/* Volume RefNum of volume which contains filter directory */
	unsigned char	fileName[32];	/* Name of file if this is an external filter			*/
	short			numMatches;		/* Number of Matches for this translator (0 means all files) */
	MatchInfo		matches[10];	/* An array of all file matches for this translator */
	unsigned char	name[32];		/* Description of translator - appears in pop-up menus */
};
typedef struct TransDescribe TransDescribe;
typedef TransDescribe *TransDescrPtr, **TransDescrHandle;

struct SFParamBlock
{
	/* The following fields must be the same in this structure and MatchTranslatorBlock */
	long			allowFlags;			/* --> Flags which indicate Translators to be used */
	long			numStandard;		/* --> How many "standard" file types */
	TransDescrPtr	standard;			/* --> Native file format information */
	OSErr			ioResult;			/* <-- Errors reported through this variable */
	short			chosenTranslator;	/* <-> On input, number of translators supplied in theChosenTranslator; on output, chosen Translator number */
	TransDescrPtr	theChosenTranslator;/* <-> On input, Pointer to list of translators to build popup menu from (UseMyTransList must be true if you supply this list); on output, pointer to the Chosen Translator */
	/* End of common fields */

	SFReply			*fileReply;			/* <-- File information is returned through this pointer */
	ResType			applicNativeType;	/* --> Application native document file type, for use when library cannot be found */
	XTNDDlgHookProcPtr	
					XTNDDlogHook;		/* --> Pointer to your SF dialog hook */
	short			currentMenuItem;	/* <-- chosen menu item (GetFile) */
	short			currentSaveItem;	/* <-- chosen menu item (PutFile) */
	Point			where;				/* --> Top left corner of dialog, if zero, center on screen */
	StringPtr		prompt;				/* --> Pascal string which is displayed on dialog */
	StringPtr		buttonTitle;		/* --> Pascal string which replaces the "Open" or "Save" button */
	StringPtr		origName;			/* --> Pascal string which is the original name in a PutFile */
	short			dialogID;			/* --> Resource number of your dialog template */
	ModalFilterProcPtr
					SFFilterProc;		/* --> Pointer to your modal dialog filter proc */
	Boolean			showAllFiles;		/* --> TRUE if you want all files to be shown */
	Boolean			useTransList;		/* --> TRUE if you are using chosenTranslator and theChosenTranslator to input a list of translators to be used in creating the popup menu */
	FileFilterProcPtr
					myFileFilter;		/* --> Pointer to your SF Filter Proc */
	long			unused[1];			/* Reserved for future use, must be zero */
};
typedef struct SFParamBlock SFParamBlock;
typedef SFParamBlock *SFParamPtr;


/* This parameter block is used to find a translator to read a file */
struct MatchTranslatorBlock
{
	/* The following fields must be the same in this structure and in ClarisFileParamBlock */
	long			allowFlags;			/* --> Flags which indicate Translators to be used */
	long			numStandard;		/* --> How many "standard" file types */
	TransDescrPtr	standard;			/* --> Array of "Native" file formats */
	OSErr			ioResult;			/* <-- IO Result from Claris library */
	short			chosenTranslator;	/* <-> On input, number of translators supplied in theChosenTranslator; on output, chosen Translator number */
	TransDescrPtr	theChosenTranslator;/* <-> On input, Pointer to list of translators to build popup menu from (UseMyTransList must be true if you supply this list); on output, pointer to the Chosen Translator */
	/* End of common fields */

	StringPtr		fileName;			/* --> File name */
	short			vRefNum;			/* --> Volume Reference Number of file */
	long			ioDirID;			/* --> Directory id of file */
	Boolean			initFlag;			/* --> TRUE if the list of translators should be rebuilt */
	Boolean			useTransList;		/* --> TRUE if you are using chosenTranslator and theChosenTranslator to input a list of translators to be used in creating the popup menu */
	TransDescrPtr	oneTrans;			/* --> If this is supplied, it will be tried instead of searching all available translators */
	long			unused[4];			/* Reserved for future use, must be zero */
};
typedef	struct MatchTranslatorBlock MatchTranslatorBlock;
typedef MatchTranslatorBlock *MatchTranslatorPtr;

struct SelectParamBlock
{
	short			translatorVersion;	/* To select by translator type */
	ResType			translatorType;		/* and version */
	long			includeFlags;		/* Include any of these translator types */
	long			excludeFlags;		/* Exclude all of these translator types */
	short			includeTrans;		/* Include translators with any of these flags set */
	short			excludeTrans;		/* Excluded translator with any of these flags set */
	StringPtr		transName;			/* Include only translators with this name */	
	long			PBVersion;			/*  MUST be ONE (1) for this version  */
	Boolean			menuSortFlag;		/* TRUE if you want the list sorted and duplicates removed as when the XTND System creates a menu in ClarisGetFile or ClarisPutFile for you */
	Boolean			unused1;			/* Reserved for future use, must be zero */
	short			unused2;			/* Reserved for future use, must be zero */
	long			unused[4];			/* Reserved for future use, must be zero */
};
typedef struct SelectParamBlock SelectParamBlock;
typedef SelectParamBlock *SelectParamPtr;


/*  Prototypes for calls to the XTND system  */

pascal	OSErr	XTNDInitTranslators(short transVersion, StringPtr xtndSystemName, StringPtr clarisFolderName);
pascal	Boolean	XTNDGetFile(SFParamPtr paramPtr);
pascal	Boolean	XTNDPutFile(SFParamPtr paramPtr);
pascal	OSErr	XTNDLoadTranslator(TransDescrPtr theTranslator, TransProcPtr *transResource);
pascal	OSErr	XTNDReleaseTranslator(TransDescrPtr theTranslator);
pascal	void	XTNDCloseTranslators(void);
pascal	OSErr	XTNDRebuildTransList(void);
pascal	OSErr	XTNDSelectTranslators(SelectParamPtr selectPtr, TransDescrHandle transHandle);
pascal	Boolean	XTNDMatchFile(MatchTranslatorPtr matchPtr);

#endif __XTND__

#endif
